/*
 * sample program to send async data
 *
 * $Id: send-async.c,v 1.1 2009/08/27 19:28:55 thomasw Exp $
 *
 * Sealevel and Seamac are registered trademarks
 * of Sealevel Systems Inc.
 *
 * This code is released under the GNU General Public License (GPL)
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>

#include <errno.h>
#include <fcntl.h>
#include <memory.h>
#include <termios.h>
#include <unistd.h>

#include <linux/types.h>

#include <sys/ioctl.h>
#include <sys/time.h>
#include <sys/types.h>

int main(int argc, char* argv[])
{
	int fd, rc, i;
	int disc = N_TTY;
	struct termios termios;
	unsigned char *buf;
	int size = 256, iterations = 5;
	char *devname = "/dev/ttyS0";

	if (argc > 1)
		devname = argv[1];
	if (argc > 2)
		size = atoi(argv[2]);
	if (argc > 3)
		iterations = atoi(argv[3]);

	printf("sending %u async bytes on %s, %i runs\n", 
						size, devname, iterations);

	buf = malloc(size);
	if (!buf) {
		printf("can't allocate buffer\n");
		return ENOMEM;
	}

	// --------------------------------------------------------------------
	// Open port
	// --------------------------------------------------------------------
	fd = open(devname, O_RDWR, 0);
	if (fd < 0) {
		printf("open on device %s failed with err=%d %s\n",
			devname, errno, strerror(errno));
		return fd;
	}

	// set N_TTY (standard async) line discipline
	rc = ioctl(fd, TIOCSETD, &disc);
	if(rc < 0) {
		printf("ERROR, can't set line discipline error=%d %s\n",
		       errno, strerror(errno));
		return rc;
	}

	
	// --------------------------------------------------------------------
	// Configure device settings
	// --------------------------------------------------------------------

	// These are standard tty ldisc settings -- make sure you set these
	// if you have problems with certain characters disapearing or being
	// transformed (0xOD->0x0A, Ox11, 0x13, etc.)
	rc = tcgetattr(fd, &termios);
	if (rc < 0) {
		printf("tcgetattr() error=%d %s\n", errno, strerror(errno));
		return rc;
	}

	termios.c_iflag = 0;
	termios.c_oflag = 0;
	termios.c_cflag = CREAD | CS8 | HUPCL | CLOCAL;
	termios.c_lflag = 0;
	termios.c_cc[VTIME] = 0;
	termios.c_cc[VMIN]  = 1;
	cfsetospeed(&termios, B9600);
	cfsetispeed(&termios, B9600);

	rc = tcsetattr(fd, TCSANOW, &termios);
	if (rc < 0) {
		printf("tcsetattr() error=%d %s\n", errno, strerror(errno));
		return rc;
	}


	// --------------------------------------------------------------------
	// TX Data
	// --------------------------------------------------------------------
	for (i = 0; i < iterations; i++) {
		int j;
		// initialize send buffer
		for (j = 0; j < size; j++)
			buf[j] = (unsigned char)j;

		printf("writing NOW\n");
		rc = write(fd, buf, size);
		if (rc < 0) {
			printf("write error=%d %s\n",
			       errno, strerror(errno));
			return rc;
		}
	
		rc = tcdrain(fd);
		printf("iteration %i complete (rc=%d)\n", (i + 1), rc);
	}

	printf("all data sent\n");
	return 0;
}

