/**
 * $Id: interface.h,v 1.1 2009/03/12 15:00:51 thomasw Exp $
 *
 * Copyright (C) 2008 Sealevel Systems Inc.
 *   
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *   
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Sealevel Systems Inc.
 * 155 Technology Place
 * P.O. Box 830
 * Liberty, SC 29657
 * www.sealevel.com
 * 
 *
 * For support please contact us by phone at (864) 843-4343 during normal
 * bussiness hours (8am - 5pm EST, M-F) or e-mail us at support@sealevel.com
 *
 */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#ifndef SEACOM_INTERFACE_H__
#define SEACOM_INTERFACE_H__

// Define a maximum number of ports for use in DeviceConfig struct
#define MAX_PORTS 	16

typedef enum INTERFACE
{
	UNDEFINED,
	RS232,
	RS422,
	RS485_2W,
	RS485_2WT,
	RS485_2WE,
	RS485_2WTE,
	RS485_4W,
	RS485_4WT,
} Interface;

// Convert a human readable interface string into a Interface enum value
static Interface atointerface(char *string)
{
	int i;

	for (i = 0; i < strlen(string); i++)
	{
		string[i] = tolower(string[i]);
	}

	// Now step through the list of possible strings
	if (strcmp(string, "232") == 0)
		return RS232;
	else if (strcmp(string, "422") == 0)
		return RS422;
	else if (strcmp(string, "485-2w") == 0)
		return RS485_2W;
	else if (strcmp(string, "485-2w-t") == 0)
		return RS485_2WT;
	else if (strcmp(string, "485-2w-e") == 0)
		return RS485_2WE;
	else if (strcmp(string, "485-2w-t-e") == 0)
		return RS485_2WTE;
	else if (strcmp(string, "485-4w") == 0)
		return RS485_4W;
	else if (strcmp(string, "485-4w-t") == 0)
		return RS485_4WT;
	else
		return UNDEFINED;
}

// Convert a enum value to a human readable string value
static char *interfaceToString(Interface interface)
{
	switch (interface)
	{
		case RS232:		return "232";
		case RS422:		return "422";
		case RS485_2W:		return "485-2w";
		case RS485_2WT:		return "485-2w-t";
		case RS485_2WE:		return "485-2w-e";
		case RS485_2WTE:	return "485-2w-t-e";
		case RS485_4W:		return "485-4w";
		case RS485_4WT:		return "485-4w-t";
		default:
		case UNDEFINED:		return "Undefined";
	}	
}

typedef struct DEVICECONFIG
{
	int model;
	int vendor;
	int numports;
	Interface interface[MAX_PORTS];
} DeviceConfig;

// Initialize a DeviceConfig structure
static void initDeviceConfig(DeviceConfig *config)
{
	int i;

	config->model = 0;
	config->numports = 0;

	for (i = 0; i < MAX_PORTS; i++)
		config->interface[i] = UNDEFINED;
}

#endif
