/**
 * $Id: slusb.h,v 1.3 2009/03/12 15:26:24 thomasw Exp $
 *
 * Copyright (C) 2008 Sealevel Systems Inc.
 *   
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *   
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Sealevel Systems Inc.
 * 155 Technology Place
 * P.O. Box 830
 * Liberty, SC 29657
 * www.sealevel.com
 * 
 *
 * For support please contact us by phone at (864) 843-4343 during normal
 * bussiness hours (8am - 5pm EST, M-F) or e-mail us at support@sealevel.com
 *
 */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "interface.h"
#include "ftd2xx/ftd2xx.h"

#ifndef SEACOM_SLUSB_H__
#define SEACOM_SLUSB_H__

typedef struct DEVICE
{
	unsigned short vendor;
	unsigned short model;
	int numports;
	int name;
} Device;


static Device supportedUSBList[] = 
{
	{0x0C52, 0xA015, 1, 2123}, 		// 2123 - 1 port
	{0x0C52, 0xA017, 1, 2128}, 		// 2128 - 1 port
	{0x0C52, 0xA016, 2, 2223}, 		// 2223 - 2 ports
	{0x0C52, 0xA018, 2, 2228}, 		// 2228 - 2 ports
	{0x0C52, 0xA01C, 2, 9106}, 		// 9106 - 2 ports
	{0x0C52, 0xA009, 4, 2423}, 		// 2423 - 4 ports
	{0x0C52, 0xA00F, 4, 2433}, 		// 2433 - 4 ports
	{0x0C52, 0xA019, 5, 9105},		// 9105 - 5 ports
	{0x0C52, 0xA00B, 8, 2823},		// 2823 - 8 ports
	{0x0C52, 0xA011, 8, 2833},		// 2833 - 8 ports
};

#define EEPROM_OFFSET 10

// This is a register mapping for use with our current Software selectable USB products
static unsigned char convertFromInterface(Interface interface)
{
	switch (interface)
	{
		case RS232:		return 0x01;
		case RS422:		return 0x1A;
		case RS485_2W:		return 0x24;
		case RS485_2WT:		return 0x34;
		case RS485_2WE:		return 0x2C;
		case RS485_2WTE:	return 0x3C;
		case RS485_4W:		return 0x0C;
		case RS485_4WT:		return 0x1C;
		case UNDEFINED:
		default:		return 0x00;
	}
}

static Interface convertToInterface(unsigned char interface)
{
	switch (interface)
	{
		case 0x01:		return RS232;
		case 0x1A:		return RS422;
		case 0x24:		return RS485_2W;
		case 0x34:		return RS485_2WT;
		case 0x2C:		return RS485_2WE;
		case 0x3C:		return RS485_2WTE;
		case 0x0C:		return RS485_4W;
		case 0x1C:		return RS485_4WT;
		default:		return UNDEFINED;
	}
}

// wrapper for FTDI open which only opens Sealevel USB devices
FT_HANDLE slusb_openDevice(int device, Device *info);

// Return the number of currently plugged in devices total
int slusb_getNumDevices();

// Return the number of currently plugged in devices of type Device
int slusb_getNumDevicesByType(Device type);

// Get the currently active electrical interfaces for the specified device
DeviceConfig slusb_getInterfaces(int device);

// Apply a device config across an entire device
int slusb_setInterfaces(int device, DeviceConfig config);
#endif
