// Basic app to read/write to 3540's config registers and
// set the mode: RS-232, RS-422, RS-485.

#include <stdio.h>
#include <stdlib.h>
#include <sys/io.h>
#include <unistd.h>

#define PORT_SIZE     0x010    //Number of IO addresses for each port (16)
#define CONFIG_OFFSET 0x008    //Offset for configuration port (BASE + 8)

// ----------------------------------------------------------------------------
// Global variable: BASE I/O address for one 4-port 3540
// ----------------------------------------------------------------------------
unsigned int BASE = 0;


// ----------------------------------------------------------------------------
// We need to know what the base IO address is set to on the card.  If the user
// gives us a bum address, we'll just keep asking until they give us a good one.
// ----------------------------------------------------------------------------
void getBase() 
{
	// This is an array of the possible valid IO address for the 3540.
	unsigned int BASES[7] = {0x300, 0x400, 0x500, 
				 0x600, 0x1500, 0x3220, 0x4220};

	if( 0 )
		; //read input file and set it
	else {
		unsigned int input;
		while( BASE == 0 ) {
			system( "clear" );
			printf( "What is your base IO address?\n" );
			scanf( "%x", &input );
			int i;
			for( i = 0; i < 7; i++ )
				if( input == BASES[i] ) BASE = input;
		}
	}
}

// ----------------------------------------------------------------------------
// Routine to change the electrical interface setting on a range of specified
// ports.  ie: Change all ports from 0-3, or just one 1-1
// ----------------------------------------------------------------------------
void setPortConfig( int start, int finish, unsigned int setting ) 
{
	iopl(3); //Give us access to ALL the IO... (Could be dangerous)

	for( ; start <= finish; start++ ) {
		int port;    //intermediate step to find the config port address
		port = BASE + CONFIG_OFFSET + (start * PORT_SIZE);

		outb( setting, port );  //write to the port config register
	}

	iopl(0); //Take away our access, we might do naughty things.
}

// ----------------------------------------------------------------------------
// We need to figure out what the user wants to do exactly.
// ----------------------------------------------------------------------------
void userselect()
{
	int port, mode;

	system("clear");
	printf("What port do you wish to configure (1-4)?\n");

	scanf( "%d", &port );
	if (port < 1 || port > 4) return;

	system("clear");
	printf( "Desired mode:\n");
	printf( "  1. RS-232\n" );
	printf( "  2. RS-422 (No termination)\n" );
	printf( "  3. RS-422 (Termination)\n" );
	printf( "  4. RS-485 (No term, no echo)\n" );
	printf( "  5. RS-485 (No term, with echo)\n" );
	printf( "  6. RS-485 (Term, no echo)\n" );
	printf( "  7. RS-485 (Term, with echo)\n" );
	printf( "\n" );
	
	scanf( "%d", &mode );
	if (mode < 1 || mode > 7) return;

	switch (mode)
	{
		case 1: mode = 0x1; break; //232
		case 2: mode = 0x4; break; //422 no term
		case 3: mode = 0x6; break; //422 with term
		case 4: mode = 0x0; break; //485 no term no echo
		case 5: mode = 0x8; break; //485 no term echo
		case 6: mode = 0x2; break; //485 term no echo
		case 7: mode = 0xA; break; //485 term echo
		default: mode = 0x00; break;
	}

	setPortConfig(port-1, port-1, mode);
}

// ----------------------------------------------------------------------------
// Time to decide what to do!
// ----------------------------------------------------------------------------
void choice( int input ) 
{
	switch( input ) 
	{
		case 1: userselect(); break;
		case 2: setPortConfig(0,3,0x1); break; //all 232
		case 3: setPortConfig(0,3,0x4); break; //all 422 no term
		case 4: setPortConfig(0,3,0x6); break; //all 422 with term
		case 5: setPortConfig(0,3,0x0); break; //all 485 no term no echo
		case 6: setPortConfig(0,3,0x8); break; //all 485 no term echo
		case 7: setPortConfig(0,3,0x2); break; //all 485 term no echo
		case 8: setPortConfig(0,3,0xA); break; //all 485 term echo
		default: break;  //ask what to do
	}
}

// ----------------------------------------------------------------------------
// Displays the main config menu.  Doesn't acutally do any port config, but
// calls the proper routine with the right parameters to do what is required.
// Should also get config file support later on, but I'm not even adding a stub.
// ----------------------------------------------------------------------------
void mainMenu() 
{
	int input = 0;

	while( input != 9 ) {

		system( "clear" );
		printf( "What would you like to do?\n" );
		printf( "  1. Configure specific port\n" );
		printf( "  2. Set all ports to RS-232\n" );
		printf( "  3. Set all ports to RS-422 (No termination)\n" );
		printf( "  4. Set all ports to RS-422 (Termination)\n" );
		printf( "  5. Set all ports to RS-485 (No term, no echo)\n" );
		printf( "  6. Set all ports to RS-485 (No term, with echo)\n" );
		printf( "  7. Set all ports to RS-485 (Term, no echo)\n" );
		printf( "  8. Set all ports to RS-485 (Term, with echo)\n" );
		printf( "  9. Quit\n" );
		printf( "\n-->");

		scanf( "%d", &input );
		// Was input valid or should you ask again
		if( input > 0 && input < 10 ) choice( input );
	}
	
	system( "clear" );  //remove all our garbage and exit
}

// ----------------------------------------------------------------------------
// This is a simple little utility to set/change the electrical interface
// on the 3540 serial card.
// ----------------------------------------------------------------------------
int main(int argc, char *argv[]) 
{
//	if (argc == 1)
//	{
		getBase();          //initialize the BASE IO address
		mainMenu();         //main loop is in here
//	}

	return 0;
}
